--------------------------------------------------------------------------------
-- Title				:	Shift Register.
-- Project			:	Sample code
-- File				:	shreg.vhd
-------------------------------------------------------------------------------
-- Organization	:	ni logic PVt. Ltd., Pune, India
-------------------------------------------------------------------------------
-- Description		:	4 bit shift register with enable, load and parallel o/ps
--						USDP has Active 'LOW' Reset
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity shr_reg is
    
		Port (
	 			clk		:	in STD_LOGIC;
				rst		:	in STD_LOGIC;
          	Sin		:	in STD_LOGIC;		-- serial i/p
				load		:	in STD_LOGIC;		-- load i/p
				en			:	in STD_LOGIC;		-- enable i/p
     			Data		:	in STD_LOGIC_VECTOR(3 downto 0);
				Q			:	out STD_LOGIC_VECTOR(3 downto 0); -- parallel o/p
     			Sout		:	out STD_LOGIC  -- serial o/p	  
		      );
end shr_reg;

architecture Behavioral of shr_reg is
Signal REG: STD_LOGIC_VECTOR(3 downto 0);
begin
process(clk,rst)
begin
	if rst='0' then
		reg	<= (others=>'0');
	elsif clk'event and clk='1' then
		if load='1' then
			reg	<= data;
		elsif en='1' then
			reg	<= sin & reg(3 downto 1);
		end if;
	end if;
end process;

-- serial o/p
sout	<= reg(0);

-- parallel o/ps
q		<= reg;

end Behavioral;