--------------------------------------------------------------------------------
-- Title				:	Universal Counter
-- Project			:	Sample code
-- File				:	unv_cntr.vhd
-------------------------------------------------------------------------------
-- Organization	:	ni logic PVt. Ltd., Pune, India
-------------------------------------------------------------------------------
-- Description		:	4-bit counter, with load, diretion control, enable and 
--							Asynchronous reset.
--						USDP has Active 'LOW' Reset
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity unv_cntr is
Port 
(
	clk 		:	in std_logic;
   rst    	: 	in std_logic;
   load   	:	in std_logic;
   updown 	:  in std_logic;
   En 		:  in std_logic;
   din 		: 	in std_logic_vector(3 downto 0);
	q		 	:  out std_logic_vector(3 downto 0)
			);
end unv_cntr;

architecture Behavioral of unv_cntr is
signal count_s : std_logic_vector(3 downto 0);
begin
		
process(clk,rst,updown,en)
begin
	if rst ='0' then
		count_s <=( others =>'0');
	elsif(clk'event and clk='1')then
		if(load='1')then	 			 -- loading the counter with data i/ps
			count_s	<=din;
		elsif en='1' then		 -- counter enable
			if updown='1'then			 -- counting up
				count_s	<= count_s + '1';
			else							 -- counting down
				count_s	<= count_s - '1';
			end if;
		end if;
	end if;
end process;

-- assigning o/p
q <=count_s;

end Behavioral;